// Copyright 2006-2007 all rights reserved; see LICENSE file for BSD-style license

package edu.berkeley.sbp;
import edu.berkeley.sbp.util.*;
import edu.berkeley.sbp.Sequence.Pos;
import edu.berkeley.sbp.Sequence.Pos;
import java.util.*;

final class Result implements GraphViz.ToGraphViz {

    private Forest f;
    private Node parent;
    private HashSet<Node> children = new HashSet<Node>();
    private boolean destroyed = false;
    private int usedByNonDoomedNode = 0;

    public GSS.Phase phase() { return parent==null ? null : parent.phase(); }
    public Forest getForest() { return f; }
    public Node parent() { return parent; }
    public void addChild(Node child) {
        children.add(child);
        usedByNonDoomedNode += child.state().doomed ? 0 : 1;
    }
    public void removeChild(Node child) {
        if (!children.contains(child)) return;
        children.remove(child);
        usedByNonDoomedNode -= child.state().doomed ? 0 : 1;
        check();
    }

    public boolean usedByAnyNode() { return children.size() > 0; }
    public boolean usedByNonDoomedNode() { return usedByNonDoomedNode > 0; }

    public String toString() { return super.toString()+"->"+parent(); }

    public void check() { if (children.size()==0) destroy(); }
    public void destroy() {
        if (destroyed) return;
        if (parent==null) return;  // never destroy the "primordeal" result
        destroyed = true;
        parent.removeChild(this);
        while(children.size() > 0)
            for(Node n : children) {
                removeChild(n);
                n.removeResult(this);
                break;
            }
    }

    public Result(Forest f, Node parent, Pos reduction) {
        this(f, parent, reduction, null);
    }
    public Result(Forest f, Node parent, Pos reduction, GSS.Phase target) {
        this.f = f;
        this.parent = parent;
        if (parent != null) parent.addChild(this);
        if (reduction == null) return;
        Parser.Table.State state0 = (Parser.Table.State)parent.state().gotoSetNonTerminals.get(reduction);
        target.newNodeFromReduction(this, state0, reduction);
    }

    // GraphViz //////////////////////////////////////////////////////////////////////////////

    public GraphViz.Node toGraphViz(GraphViz gv) {
        if (gv.hasNode(this)) return gv.createNode(this);
        GraphViz.Node n = gv.createNode(this);
        n.label = ""+f;
        n.shape = "rectangle";
        if (parent()!=null) n.edge(parent, "");
        n.color = "blue";
        if (phase() != null)
            ((GraphViz.Group)phase().toGraphViz(gv)).add(n);
        return n;
    }
    public boolean isTransparent() { return false; }
    public boolean isHidden() { return false; }

}